﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Orchard.Test
{
    public class Blog : UIHelperBase
    {
        public string Title { get; private set; }

        public string Permalink { get; private set; }

        public string MenuText { get; private set; }

        public string Owner { get; private set; }

        public Blog(TestLibrary settings, string title,
            string permalink, string menutext, string owner)
            : base(settings)
        {
            Title = title;
            Permalink = permalink;
            MenuText = menutext;
            Owner = owner;
        }

        public void ManagePost()
        {
            var blogAdminUrl = String.Format("/Admin/Blogs/{0}", Permalink);
            selenium.Open(blogAdminUrl);
            selenium.WaitForPageToLoad(TestLibrary.Consts.TimeToWaitForPageToLoad);
        }

        public int GetBlogIndexOnManageBlogPage()
        {
            var count = (int)selenium.GetXpathCount("//div[@id='content']/div[@id='main']/ul/li");
            var index = 1;
            for (; index <= count; ++index)
            {
                string path = string.Format("//div[@id='content']/div[@id='main']/ul/li[{0}]/div/div[2]/h3/a", index);
                var text = selenium.GetText(path);
                if (string.CompareOrdinal(Title, text) == 0)
                    break;
            }

            if (index > count)
                throw new CaseFailureException(string.Format("Can't find blog {0} in manage blogs page!", Title));

            return index;
        }
    }

    public class BlogHelper : UIHelperBase
    {
        public BlogHelper(TestLibrary settings)
            : base(settings)
        {
        }

        public Blog CreateRandomBlog()
        {
            var uniqueIdentifier = UIHelperBase.GenerateUniqueIdentifier();
            var title = string.Format("Blog {0}", uniqueIdentifier);
            var menuText = title;

            return CreateBlog(title, menuText);
        }

        public Blog CreateBlog(string title,  string menuText)
        {
            return CreateBlog(title,  menuText, string.Empty);
        }

        public Blog CreateBlog(string title,  string menuText, string description)
        {
            return CreateBlog(title,  menuText, description, true);
        }

        public Blog CreateBlog(string title, string menuText, string description, bool addMainMenu)
        {
            return CreateBlog(title, string.Format("blog-{0}", UIHelperBase.GenerateUniqueIdentifier()), menuText, description, true, null);
        }

        public Blog CreateBlog(string title, string permalink,string menuText,
            string description, bool addMainMenu, string owner)
        {
            ManageBlogs();
            // empty string is intendly left for testing purpose
            if (string.IsNullOrEmpty(title))
                throw new CaseErrorException(new ArgumentNullException("title"));
            if (string.IsNullOrEmpty(menuText))
                throw new CaseErrorException(new ArgumentNullException("menuText"));
            if (!string.IsNullOrEmpty(owner))
                throw new CaseErrorException("Set owner is not implemented yet!");
       
            selenium.Click("link=Add New Blog");
            selenium.WaitForPageToLoad(TestLibrary.Consts.TimeToWaitForPageToLoad);
            selenium.Type("Routable_Title", title);
            selenium.Type("MenuText", menuText);

            if (!string.IsNullOrEmpty(description))
                selenium.Type("Description", description);
            if (!string.IsNullOrEmpty(permalink))
                selenium.Type("Routable_Slug", permalink);
            if (addMainMenu)
                selenium.Click("OnMainMenu");

            selenium.Click("//input[@value='Add']");
            selenium.WaitForPageToLoad(TestLibrary.Consts.TimeToWaitForPageToLoad);

            return new Blog(TestSettings, title, permalink, menuText, owner);
        }

        public void ManageBlogs()
        {
            selenium.Open("/Admin/Blogs");
            selenium.WaitForPageToLoad(TestLibrary.Consts.TimeToWaitForPageToLoad);
        }

        public void DeleteBlog(Blog blog)
        {
            ManageBlogs();
            int index = blog.GetBlogIndexOnManageBlogPage();
            selenium.Click(String.Format("//div[@id='content']/div[@id='main']/ul/li[{0}]/div/div[1]/form/button", index));
            selenium.WaitForPageToLoad(TestLibrary.Consts.TimeToWaitForPageToLoad);
        }

        public void EditBlog(Blog blog)
        {
            ManageBlogs();
            selenium.Click(String.Format("link={0}",blog.Title));
            selenium.WaitForPageToLoad(TestLibrary.Consts.TimeToWaitForPageToLoad);
            selenium.Click("link=Edit");
            selenium.WaitForPageToLoad(TestLibrary.Consts.TimeToWaitForPageToLoad);
            selenium.Click("//input[@value='Save']");
            selenium.WaitForPageToLoad(TestLibrary.Consts.TimeToWaitForPageToLoad);
        }      
    }
}
