﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Selenium;

namespace Orchard.Test
{
    public class TestLibrary
    {
        public UserHelper UserHelper { get; private set; }

        public BlogHelper BlogHelper { get; private set; }

        public ISelenium Selenium { get; private set; }

        public string SiteUrl { get; private set; }

        public class Consts
        {
            public const string TimeToWaitForPageToLoad = "30000";

            public const string AdminUserName = "administrator";

            public const string ContributorUser = "Contributor1";

            public const string AuthorUser = "Author1";

            public const string ModeratorUser = "Moderator1";

            public const string EditorUser = "Editor1";

            public const string CommonPassword = "0123456";

            public static readonly string DefaultSeleniumHost;

            public static readonly int DefaultSeleniumPort;

            public static readonly string DefaultBrowser;

            public static readonly string DefaultOrchardSite;

            public const string OrchardPhysicalPath = @"E:\临时文档\orchard\orchard_1fbd1352f01d\src\Orchard.Web";

            public static readonly string SqliteDbFile = string.Format(@"{0}\App_Data\Sites\Default\Orchard.db", OrchardPhysicalPath);

            static Consts()
            {
                var value = Environment.GetEnvironmentVariable("ORCHARDSITE");
                DefaultOrchardSite = string.IsNullOrEmpty(value) ? "http://localhost:30320" : value;
                value = Environment.GetEnvironmentVariable("SELENIUMPORT");
                DefaultSeleniumPort = string.IsNullOrEmpty(value) ? 4444 : Int32.Parse(value);
                value = Environment.GetEnvironmentVariable("SELENIUMHOST");
                DefaultSeleniumHost = string.IsNullOrEmpty(value) ? "localhost" : value;
                value = Environment.GetEnvironmentVariable("BROWSERMODE");
                DefaultBrowser = string.IsNullOrEmpty(value) ? "*firefox" : value;
            }
        }

        public TestLibrary(ISelenium selenium)
        {
            this.UserHelper = new UserHelper(this);
            this.BlogHelper = new BlogHelper(this);
            Selenium = selenium;
        }

        public static TestLibrary SetupTest()
        {
            return SetupTest(Consts.DefaultSeleniumHost, Consts.DefaultSeleniumPort,
                Consts.DefaultBrowser, Consts.DefaultOrchardSite);
        }

        public static TestLibrary SetupTest(string seleniumHost, int seleniumPort,
            string browser, string orchardSite)
        {
            var selenium = new DefaultSelenium(
                seleniumHost, seleniumPort, browser, orchardSite);
            selenium.Start();

            return new TestLibrary(selenium) { SiteUrl = orchardSite };
        }

        public void Shutdown()
        {
            try
            {
                Selenium.Stop();
            }
            catch (Exception)
            {
                // Ignore errors if unable to close the browser
            }
        }
    }
}
