﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Selenium;

namespace Orchard.Test
{
    public abstract class UIHelperBase
    {
        protected TestLibrary TestSettings { get; private set; }
        protected ISelenium selenium { get { return TestSettings.Selenium; } }

        protected UIHelperBase(TestLibrary settings)
        {
            TestSettings = settings;
        }

        public static string GenerateUniqueIdentifier(DateTime date)
        {
            return string.Format("{0}{1}{2}{3}{4}{5}",
                                date.Year, date.Month, date.Day, date.Hour, date.Minute, date.Second);
        }

        protected static string GenerateUniqueIdentifier()
        {
            return GenerateUniqueIdentifier(DateTime.Now);
        }
    }

    public class CaseFailureException : Exception
    {
        public CaseFailureException() { }

        public CaseFailureException(string message)
            : this(message, null)
        {
        }

        public CaseFailureException(Exception inner)
            : base("A test failure occurs, possibly caused by product bugs, please investigate it! you can check inner exception for details!", inner)
        {
        }

        public CaseFailureException(string message, Exception inner)
            : base(string.Format("A test failure occurs, possibly caused by product bugs, please investigate it!\ndetailed message: \n{0}", message), inner)
        {
        }
    }

    public class CaseErrorException : Exception
    {
        public CaseErrorException()
        {
        }

        public CaseErrorException(string message)
            : this(message, null)
        {
        }

        public CaseErrorException(Exception inner)
            : base("A test error occurs, which indicates errors in test code, please check inner exception for details!", inner)
        {
        }

        public CaseErrorException(string message, Exception inner)
            : base(string.Format("A test error occurs, please check message: {0}", message), inner)
        {
        }
    }  

    public class UserHelper : UIHelperBase
    {
        public UserHelper(TestLibrary settings)
            : base(settings)
        {
        }

        public void LogOnAsAdmin()
        {
            LogOn(TestLibrary.Consts.AdminUserName, TestLibrary.Consts.CommonPassword);
        }

        public void LogOn(string username, string password)
        {
            if (String.IsNullOrEmpty(username))
            {
                throw new CaseErrorException(new ArgumentNullException("username"));
            }

            if (String.IsNullOrEmpty(password))
            {
                throw new CaseErrorException(new ArgumentNullException("password"));
            }

            selenium.Open("/");
            if (selenium.IsElementPresent("link=Log Off"))
            {
                selenium.Click("link=Log Off");
                selenium.WaitForPageToLoad(TestLibrary.Consts.TimeToWaitForPageToLoad);
            }

            if (selenium.IsElementPresent("link=Log On"))
            {
                selenium.Click("link=Log On");
            }

            if (selenium.IsElementPresent("link=Login"))
            {
                selenium.Click("link=Login");
            }

            selenium.WaitForPageToLoad(TestLibrary.Consts.TimeToWaitForPageToLoad);
            selenium.Type("username", username);
            selenium.Type("password", password);
            selenium.Click("//input[@value='Log On']");
            selenium.WaitForPageToLoad(TestLibrary.Consts.TimeToWaitForPageToLoad);
        }

        public void LogOff()
        {
            selenium.Open("/");
            if (selenium.IsElementPresent("link=Log Off"))
            {
                selenium.Click("link=Log Off");
                selenium.WaitForPageToLoad(TestLibrary.Consts.TimeToWaitForPageToLoad);
            }
        }

        public bool IsLoginFaled()
        {
            selenium.Open("/");
            return selenium.IsTextPresent("Log On");
        }
    }
}
