﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using NModel.Attributes;
using NModel;

namespace OrchardMode
{
    public enum Password { Correct, Incorrect }

    public enum LoginStatus { Success, Failure }

    [Feature("Login")]
    public static class Login
    {
        public static Map<User, LoginStatus> ActiveLoginRequests = Map<User, LoginStatus>.EmptyMap;

        [Requirement("Send username and password to the server to log in.")]
        [Action]
        public static void Login_Start(User user, Password password)
        {
            if (password == Password.Correct)
                ActiveLoginRequests = ActiveLoginRequests.Add(user, LoginStatus.Success);
            else
                ActiveLoginRequests = ActiveLoginRequests.Add(user, LoginStatus.Failure);
        }

        public static bool Login_StartEnabled()
        {
            return WebSiteModel.State == ModeState.Running;
        }

        public static bool Login_StartEnabled(User user)
        {
            return !ActiveLoginRequests.ContainsKey(user) &&
                   !WebSiteModel.UsersLoggedIn.Contains(user);
        }

        [Requirement("Web server returns login response to browser.")]
        [Action]
        public static void Login_Finish(User user, LoginStatus status)
        {
            if (status == LoginStatus.Success)
            {
                WebSiteModel.UsersLoggedIn = WebSiteModel.UsersLoggedIn.Add(user);
            }
            else // if status == LoginStatus.Failure
            {
                if (WebSiteModel.UsersLoggedIn.Contains(user))
                    WebSiteModel.UsersLoggedIn = WebSiteModel.UsersLoggedIn.Remove(user);
            }

            ActiveLoginRequests = ActiveLoginRequests.RemoveKey(user);
        }
        public static bool Login_FinishEnabled()
        {
            return WebSiteModel.State == ModeState.Running;
        }

        [Requirement("")]
        public static bool Login_FinishEnabled(User user, LoginStatus status)
        {
            return ActiveLoginRequests.ContainsKey(user) && ActiveLoginRequests[user].Equals(status);
        }

        [Requirement("It should be possible to log out from any page")]
        [Action]
        public static void Logout(User user)
        {
            WebSiteModel.UsersLoggedIn = WebSiteModel.UsersLoggedIn.Remove(user);
        }
        public static bool LogoutEnabled()
        {
            return WebSiteModel.State == ModeState.Running;
        }

        public static bool LogoutEnabled(User user)
        {
            return WebSiteModel.UsersLoggedIn.Contains(user);
        }
    }
}
